(* 3 modules utilitaires et le type des graphes *)
module IntSet = Set.Make (Int)
module IntMap = Map.Make (Int)
module StrMap = Map.Make (String)

type graph = {
  vertices : IntSet.t; (* ensemble des sommets *)
  edges : int list IntMap.t;
      (* dictionnaire sommet => liste des sommets voisins. Attention,
         si un sommet n'a pas de voisin, il n'apparait pas comme clé
         dans ce dictionnaire (il n'est pas automatiquement associé à []) *)
  labels : string IntMap.t;
      (* dictionnaire sommet => nom de la page (pour l'affichage) *)
}

module WikiGraph : sig
  include
    Graph.Path.G with type V.t = int and type E.t = int * int and type t = graph

  (* on inclut la signature Graph.Path.G en précisant des égalité de types *)

  val build : string -> string -> t
  (* on étend la signature avec une fonction de construction, qui prend 2 noms
     de fichiers et renvoie le graphe *)
end = struct
  (* Premier sous-module représentant les sommets *)
  module V = struct
    type t = int

    let compare = Int.compare
    let hash = Hashtbl.hash
    let equal a b = a = b
  end

  (* Deuxième sous-module représentant les arrêtes *)
  module E = struct
    type t = int * int
    (* les arrêtes sont de simple paires de sommets *)

    (* On n'utilise pas les étiquettes d'arrêtes dans nos graphes, mais il
       faut quand même définir un type pour respecter la signature.
       On utilise le type constant unit *)
    type label = unit

    let label _ = ()
    let src e = fst e
    let dst e = snd e
    let create v1 () v2 = (v1, v2)
  end

  type t = graph
  (* on dit que le type t du module est exactement le type graph défini au dessus *)

  (** [iter_vertex f g] appelle la fonction [f] tour a tour sur tous les sommets du graphe [g]*)
  let iter_vertex f g = failwith "TODO"

  (** [fold_vertex f g] calcule
      [(f vn .... (f v2 (f v1 acc)))] où les [vi] sont les sommets du graphe.
  *)
  let fold_vertex f g acc = failwith "TODO"

  (** [iter_succ f g v] appelle [f] tour à tour sur tous les voisins de [v] *)
  let iter_succ f g v = failwith "TODO"

  (** [iter_succ_e f g v] appelle [f] tour à tour sur toutes les arrêtes de [(v,w)] où
      [w] est un voisin de [v] *)
  let iter_succ_e f g v = failwith "TODO"

  (** [fold_edges_e f g acc]  calcule
      [(f em .... (f e2 (f e1 acc)))] où les [ei] sont les arrêtes du graphe.
      *)
  let fold_edges_e f g acc = failwith "TODO"

  (** [nb_vertex g] renvoie le nombre de sommets *)
  let nb_vertex g = failwith "TODO"

  (** construit le graphe, ne pas modifier *)
  let build f1 f2 =
    let c_in = open_in f1 in
    let rec loop vertices labels =
      match In_channel.input_line c_in with
      | None -> (vertices, labels)
      | Some sv -> (
          let v = int_of_string sv in
          match In_channel.input_line c_in with
          | None -> assert false
          | Some lab -> loop (IntSet.add v vertices) (IntMap.add v lab labels))
    in
    let vertices, labels = loop IntSet.empty IntMap.empty in
    let () = close_in c_in in
    let c_in = open_in f2 in
    let rec loop edges =
      match In_channel.input_line c_in with
      | None -> edges
      | Some s -> (
          match List.map int_of_string (String.split_on_char ' ' s) with
          | [] -> assert false
          | v :: others ->
              if IntSet.mem v vertices then loop (IntMap.add v others edges)
              else loop edges)
    in
    let edges = loop IntMap.empty in
    let () = close_in c_in in
    { vertices; edges; labels }
end

(* Le module représentant des poids *)
(* Reformuler ce module pour utiliser Int et ne pas ré-écrire le type t et les fonctions
   compare, add et zero qui sont dans le module Int *)
module Weight = struct
  type edge = WikiGraph.E.t
  (* une arrête est telle que définie ci-dessus, i.e. une paire d'entiers *)

  type t = int (* nos poids sont entiers *)

  let weight _ = 1 (* le poid d'une arrête vaut toujours 1 *)
  let compare w1 w2 = Int.compare w1 w2 (* comparer 2 poids *)
  let add w1 w2 = w1 + w2 (* additionner 2 poids *)
  let zero = 0 (* élément neutre pour l'addition *)
end
